@@ -444,6 +444,50 @@ static LRESULT vboxClipboardWinProcessMsg(PSHCLCONTEXT pCtx, HWND hwnd, UINT msg
                                         cb = (uint32_t)(cbActual + 2);
                                     }
                                 }
+                                else if (fFormat == VBOX_SHCL_FMT_HTML)
+                                {
+                                    /* Wrap content into CF_HTML clipboard format if needed. */
+                                    if (!SharedClipboardWinIsCFHTML((const char *)pMem))
+                                    {
+                                        char *pszWrapped = NULL;
+                                        uint32_t cbWrapped = 0;
+                                        rc =  SharedClipboardWinConvertMIMEToCFHTML((const char *)pMem, cb, &pszWrapped, &cbWrapped);
+                                        if (RT_SUCCESS(rc))
+                                        {
+                                            if (GlobalUnlock(hMem) == 0)
+                                            {
+                                                hMem = GlobalReAlloc(hMem, cbWrapped, 0);
+                                                if (hMem)
+                                                {
+                                                    pMem = GlobalLock(hMem);
+                                                    if (pMem)
+                                                    {
+                                                        /* Copy wrapped content back to memory passed to system clipboard. */
+                                                        memcpy(pMem, pszWrapped, cbWrapped);
+                                                        cb = cbWrapped;
+                                                    }
+                                                    else
+                                                    {
+                                                        LogRel(("Shared Clipboard: Cannot lock memory, "
+                                                                "HTML clipboard data won't be converted into CF_HTML clipboard format\n"));
+                                                        GlobalFree(hMem);
+                                                        hMem = NULL;
+                                                    }
+                                                }
+                                                else
+                                                    LogRel(("Shared Clipboard: Cannot re-allocate memory, "
+                                                            "HTML clipboard data won't be converted into CF_HTML clipboard format\n"));
+                                            }
+                                            else
+                                                LogRel(("Shared Clipboard: Cannot unlock memory, "
+                                                        "HTML clipboard data won't be converted into CF_HTML clipboard format\n"));
+
+                                            RTMemFree(pszWrapped);
+                                        }
+                                        else
+                                            LogRel(("Shared Clipboard: Cannot convert HTML clipboard data into CF_HTML clipboard format, rc=%Rrc\n", rc));
+                                    }
+                                }
                             }
 
                             if (hMem)
@@ -545,11 +589,10 @@ static LRESULT vboxClipboardWinProcessMsg(PSHCLCONTEXT pCtx, HWND hwnd, UINT msg
 
             const SHCLFORMAT fFormat = (uint32_t)pEvent->u.fReadData;
 
-            HANDLE hClip = NULL;
-
             LogFlowFunc(("SHCL_WIN_WM_READ_DATA: fFormat=%#x\n", fFormat));
 
             int rc = SharedClipboardWinOpen(hwnd);
+            HANDLE hClip = NULL;
             if (RT_SUCCESS(rc))
             {
                 if (fFormat & VBOX_SHCL_FMT_BITMAP)
@@ -565,9 +608,7 @@ static LRESULT vboxClipboardWinProcessMsg(PSHCLCONTEXT pCtx, HWND hwnd, UINT msg
                             GlobalUnlock(hClip);
                         }
                         else
-                        {
                             hClip = NULL;
-                        }
                     }
                 }
                 else if (fFormat & VBOX_SHCL_FMT_UNICODETEXT)
@@ -584,9 +625,7 @@ static LRESULT vboxClipboardWinProcessMsg(PSHCLCONTEXT pCtx, HWND hwnd, UINT msg
                             GlobalUnlock(hClip);
                         }
                         else
-                        {
                             hClip = NULL;
-                        }
                     }
                 }
                 else if (fFormat & VBOX_SHCL_FMT_HTML)
@@ -597,30 +636,45 @@ static LRESULT vboxClipboardWinProcessMsg(PSHCLCONTEXT pCtx, HWND hwnd, UINT msg
                         hClip = GetClipboardData(format);
                         if (hClip != NULL)
                         {
-                            LPVOID lp = GlobalLock(hClip);
-                            if (lp != NULL)
+                            LPVOID const pvClip = GlobalLock(hClip);
+                            if (pvClip != NULL)
                             {
-                                rc = VbglR3ClipboardWriteDataEx(&pEvent->cmdCtx, fFormat, lp, (uint32_t)GlobalSize(hClip));
+                                uint32_t const cbClip = (uint32_t)GlobalSize(hClip);
+
+                                /* Unwrap clipboard content from CF_HTML format if needed. */
+                                if (SharedClipboardWinIsCFHTML((const char *)pvClip))
+                                {
+                                    char        *pszBuf = NULL;
+                                    uint32_t    cbBuf   = 0;
+                                    rc = SharedClipboardWinConvertCFHTMLToMIME((const char *)pvClip, cbClip, &pszBuf, &cbBuf);
+                                    if (RT_SUCCESS(rc))
+                                    {
+                                        rc = VbglR3ClipboardWriteDataEx(&pEvent->cmdCtx, fFormat, pszBuf, cbBuf);
+                                        RTMemFree(pszBuf);
+                                    }
+                                    else
+                                        rc = VbglR3ClipboardWriteDataEx(&pEvent->cmdCtx, fFormat, pvClip, cbClip);
+                                }
+                                else
+                                    rc = VbglR3ClipboardWriteDataEx(&pEvent->cmdCtx, fFormat, pvClip, cbClip);
 
                                 GlobalUnlock(hClip);
                             }
                             else
-                            {
                                 hClip = NULL;
-                            }
                         }
                     }
                 }
-                if (hClip == NULL)
-                {
-                    LogFunc(("SHCL_WIN_WM_READ_DATA: hClip=NULL, lastError=%ld\n", GetLastError()));
 
-                    /* Requested clipboard format is not available, send empty data. */
-                    VbglR3ClipboardWriteData(pCtx->CmdCtx.idClient, VBOX_SHCL_FMT_NONE, NULL, 0);
-                }
+                if (hClip == NULL)
+                    LogFunc(("SHCL_WIN_WM_READ_DATA: hClip=NULL, lastError=%ld\n", GetLastError()));
 
                 SharedClipboardWinClose();
             }
+
+            /* If the requested clipboard format is not available, we must send empty data. */
+            if (hClip == NULL)
+                VbglR3ClipboardWriteDataEx(&pEvent->cmdCtx, VBOX_SHCL_FMT_NONE, NULL, 0);
             break;
         }
 
