@@ -666,6 +666,7 @@ static VOID vboxDispIfOpEnd(VBOXDISPIF_OP *pOp)
  * that knows nothing about us */
 DWORD VBoxDispIfInit(PVBOXDISPIF pDispIf)
 {
+    /* Note: NT4 is handled implicitly by VBoxDispIfSwitchMode(). */
     VBoxDispIfSwitchMode(pDispIf, VBOXDISPIF_MODE_XPDM, NULL);
 
     return NO_ERROR;
@@ -1775,6 +1776,20 @@ BOOL VBoxDispIfResizeDisplayWin7(PCVBOXDISPIF const pIf, uint32_t cDispDef, cons
     const VMMDevDisplayDef *pDispDef;
     uint32_t i;
 
+    /* SetDisplayConfig assumes the top-left corner of a primary display at (0, 0) position */
+    const VMMDevDisplayDef* pDispDefPrimary = NULL;
+
+    for (i = 0; i < cDispDef; ++i)
+    {
+        pDispDef = &paDispDef[i];
+
+        if (pDispDef->fDisplayFlags & VMMDEV_DISPLAY_PRIMARY)
+        {
+            pDispDefPrimary = pDispDef;
+            break;
+        }
+    }
+
     VBOXDISPIF_OP Op;
     DWORD winEr = vboxDispIfOpBegin(pIf, &Op);
     if (winEr != ERROR_SUCCESS)
@@ -1870,8 +1885,8 @@ BOOL VBoxDispIfResizeDisplayWin7(PCVBOXDISPIF const pIf, uint32_t cDispDef, cons
 
                 if (pDispDef->fDisplayFlags & VMMDEV_DISPLAY_ORIGIN)
                 {
-                    pSrcMode->position.x = pDispDef->xOrigin;
-                    pSrcMode->position.y = pDispDef->yOrigin;
+                    pSrcMode->position.x = pDispDef->xOrigin - (pDispDefPrimary ? pDispDefPrimary->xOrigin : 0);
+                    pSrcMode->position.y = pDispDef->yOrigin - (pDispDefPrimary ? pDispDefPrimary->yOrigin : 0);
                 }
 
                 if (pDispDef->fDisplayFlags & VMMDEV_DISPLAY_BPP)
@@ -1974,8 +1989,8 @@ BOOL VBoxDispIfResizeDisplayWin7(PCVBOXDISPIF const pIf, uint32_t cDispDef, cons
 
                 if (pDispDef->fDisplayFlags & VMMDEV_DISPLAY_ORIGIN)
                 {
-                    pSrcModeInfo->sourceMode.position.x = pDispDef->xOrigin;
-                    pSrcModeInfo->sourceMode.position.y = pDispDef->yOrigin;
+                    pSrcModeInfo->sourceMode.position.x = pDispDef->xOrigin - (pDispDefPrimary ? pDispDefPrimary->xOrigin : 0);
+                    pSrcModeInfo->sourceMode.position.y = pDispDef->yOrigin - (pDispDefPrimary ? pDispDefPrimary->yOrigin : 0);
                 }
 
                 /* Configure the path information. */
@@ -2581,6 +2596,11 @@ DWORD VBoxDispIfSwitchMode(PVBOXDISPIF pIf, VBOXDISPIF_MODE enmMode, VBOXDISPIF_
     if (enmMode == pIf->enmMode)
         return NO_ERROR;
 
+    /* Make sure that we never try to run anything else but VBOXDISPIF_MODE_XPDM_NT4 on NT4 guests.
+     * Anything else will get us into serious trouble. */
+    if (RTSystemGetNtVersion() < RTSYSTEM_MAKE_NT_VERSION(5, 0, 0))
+        enmMode = VBOXDISPIF_MODE_XPDM_NT4;
+
 #ifdef VBOX_WITH_WDDM
     if (pIf->enmMode >= VBOXDISPIF_MODE_WDDM)
     {
