@@ -1674,7 +1674,7 @@ typedef enum VBOXCAPS_ENTRY_ACSTATE
 struct VBOXCAPS_ENTRY;
 struct VBOXCAPS;
 
-typedef DECLCALLBACKPTR(void, PFNVBOXCAPS_ENTRY_ON_ENABLE)(struct VBOXCAPS *pConsole, struct VBOXCAPS_ENTRY *pCap, BOOL fEnabled);
+typedef DECLCALLBACKPTR(void, PFNVBOXCAPS_ENTRY_ON_ENABLE,(struct VBOXCAPS *pConsole, struct VBOXCAPS_ENTRY *pCap, BOOL fEnabled));
 
 typedef struct VBOXCAPS_ENTRY
 {
